@@ -1,24 +1,8 @@
-mod complex;
-mod fraction;
-mod functions;
-mod graph;
-mod help;
-mod load_vars;
-mod math;
-mod misc;
-mod options;
-mod parse;
-mod print;
-#[cfg(test)]
-mod tests;
-mod units;
-//make dependencys optional via features
-//add symbolica as an optional feature
-//rayon-ize graphs, and maybe area
-//fix ||x|-1|
-//lib, gui
-use crate::{
-    complex::NumStr,
+use crossterm::{
+    cursor::{DisableBlinking, EnableBlinking},
+    execute, terminal,
+};
+use kalc_lib::{
     functions::{functions_with_args, options_list, units_list},
     graph::graph,
     help::help_for,
@@ -31,250 +15,20 @@ use crate::{
     options::{arg_opts, commands, equal_to, file_opts, silent_commands},
     parse::input_var,
     print::{print_answer, print_concurrent},
-    AngleType::Radians,
-    Notation::Normal,
+    units::{Auto, Colors, Data, HowGraphing, Options, Variable},
 };
-use crossterm::{
-    cursor::{DisableBlinking, EnableBlinking},
-    execute, terminal,
-};
-use rug::Complex;
 use std::{
     cmp::Ordering,
+    env,
     env::args,
     fs::{File, OpenOptions},
-    io::{stdin, stdout, BufRead, BufReader, IsTerminal, Stdout, Write},
+    io::{BufRead, BufReader, IsTerminal, Stdout, Write, stdin, stdout},
+    path::{Path, PathBuf},
+    process::{Command, Stdio},
+    thread,
     thread::JoinHandle,
     time::Instant,
 };
-#[derive(Clone)]
-pub struct Variable
-{
-    name: Vec<char>,
-    parsed: Vec<NumStr>,
-    unparsed: String,
-    funcvars: Vec<(String, Vec<NumStr>)>,
-}
-#[derive(Clone, PartialEq, Copy)]
-pub struct Units
-{
-    second: f64,
-    meter: f64,
-    kilogram: f64,
-    ampere: f64,
-    kelvin: f64,
-    mole: f64,
-    candela: f64,
-    angle: f64,
-    byte: f64,
-    usd: f64,
-    unit: f64,
-}
-#[derive(Clone, PartialEq)]
-pub struct Number
-{
-    number: Complex,
-    units: Option<Units>,
-}
-#[derive(Clone)]
-pub struct Colors
-{
-    text: String,
-    prompt: String,
-    imag: String,
-    sci: String,
-    units: String,
-    brackets: Vec<String>,
-    recol: Vec<String>,
-    imcol: Vec<String>,
-    label: (String, String, String),
-    graphtofile: String,
-    default_units: Vec<(String, Number)>,
-}
-impl Default for Colors
-{
-    fn default() -> Self
-    {
-        Self {
-            text: "\x1b[0m".to_string(),
-            prompt: "\x1b[94m".to_string(),
-            imag: "\x1b[93m".to_string(),
-            sci: "\x1b[92m".to_string(),
-            units: "\x1b[96m".to_string(),
-            brackets: vec![
-                "\x1b[91m".to_string(),
-                "\x1b[92m".to_string(),
-                "\x1b[93m".to_string(),
-                "\x1b[94m".to_string(),
-                "\x1b[95m".to_string(),
-                "\x1b[96m".to_string(),
-            ],
-            recol: vec![
-                "#ff5555".to_string(),
-                "#5555ff".to_string(),
-                "#ff55ff".to_string(),
-                "#55ff55".to_string(),
-                "#55ffff".to_string(),
-                "#ffff55".to_string(),
-            ],
-            imcol: vec![
-                "#aa0000".to_string(),
-                "#0000aa".to_string(),
-                "#aa00aa".to_string(),
-                "#00aa00".to_string(),
-                "#00aaaa".to_string(),
-                "#aaaa00".to_string(),
-            ],
-            label: ("x".to_string(), "y".to_string(), "z".to_string()),
-            graphtofile: String::new(),
-            default_units: Vec::new(),
-        }
-    }
-}
-#[derive(Copy, Clone, PartialEq)]
-pub enum AngleType
-{
-    Radians,
-    Degrees,
-    Gradians,
-}
-#[derive(Copy, Clone, PartialEq)]
-pub enum Auto
-{
-    True,
-    False,
-    Auto,
-}
-#[derive(Default, Copy, Clone, PartialEq)]
-pub struct HowGraphing
-{
-    graph: bool,
-    x: bool,
-    y: bool,
-}
-#[derive(Copy, Clone, PartialEq)]
-pub struct Fractions
-{
-    num: bool,
-    vec: bool,
-    mat: bool,
-}
-#[derive(Copy, Clone, PartialEq)]
-pub enum Notation
-{
-    Normal,
-    Scientific,
-    LargeEngineering,
-    SmallEngineering,
-}
-#[derive(Copy, Clone, PartialEq)]
-pub enum GraphType
-{
-    Normal,
-    Domain,
-    DomainAlt,
-    Flat,
-    Depth,
-    None,
-}
-#[derive(Clone, Copy)]
-pub struct Options
-{
-    notation: Notation,
-    angle: AngleType,
-    graphtype: GraphType,
-    base: (i32, i32),
-    ticks: (f64, f64, f64),
-    onaxis: bool,
-    polar: bool,
-    frac: Fractions,
-    real_time_output: bool,
-    decimal_places: usize,
-    color: Auto,
-    prompt: bool,
-    comma: bool,
-    prec: u32,
-    graph_prec: u32,
-    xr: (f64, f64),
-    yr: (f64, f64),
-    zr: (f64, f64),
-    vxr: (f64, f64),
-    vyr: (f64, f64),
-    vzr: (f64, f64),
-    samples_2d: usize,
-    samples_3d: (usize, usize),
-    point_style: isize,
-    lines: Auto,
-    multi: bool,
-    tabbed: bool,
-    allow_vars: bool,
-    debug: bool,
-    slowcheck: u128,
-    interactive: bool,
-    surface: bool,
-    scale_graph: bool,
-    stay_interactive: bool,
-    graph_cli: bool,
-    units: bool,
-    si_units: bool,
-    window_size: (usize, usize),
-    keep_zeros: bool,
-    progress: bool,
-    keep_data_file: bool,
-}
-impl Default for Options
-{
-    fn default() -> Self
-    {
-        Self {
-            notation: Normal,
-            angle: Radians,
-            graphtype: GraphType::Normal,
-            base: (10, 10),
-            ticks: (16.0, 16.0, 16.0),
-            onaxis: true,
-            polar: false,
-            frac: Fractions {
-                num: true,
-                vec: true,
-                mat: false,
-            },
-            real_time_output: true,
-            decimal_places: 12,
-            color: Auto::Auto,
-            prompt: true,
-            comma: false,
-            prec: 512,
-            graph_prec: 128,
-            xr: (-8.0, 8.0),
-            yr: (-8.0, 8.0),
-            zr: (-8.0, 8.0),
-            vxr: (0.0, 0.0),
-            vyr: (0.0, 0.0),
-            vzr: (0.0, 0.0),
-            samples_2d: 8192,
-            samples_3d: (256, 256),
-            point_style: 0,
-            lines: Auto::Auto,
-            multi: true,
-            tabbed: false,
-            allow_vars: true,
-            debug: false,
-            slowcheck: 256,
-            interactive: true,
-            surface: false,
-            scale_graph: false,
-            stay_interactive: false,
-            graph_cli: false,
-            units: true,
-            si_units: false,
-            window_size: (0, 0),
-            keep_zeros: false,
-            progress: false,
-            keep_data_file: false,
-        }
-    }
-}
 fn main()
 {
     let mut colors = Colors::default();
@@ -480,14 +234,14 @@ fn main()
                                             add_var(le, r, i, &mut vars, options, true, true, true)
                                         {
                                             err = true;
-                                            println!("\x1b[G\x1b[K{}", s)
+                                            println!("\x1b[G\x1b[K{s}")
                                         }
                                     }
                                     else if let Err(s) =
                                         add_var(le, r, i, &mut vars, options, true, true, false)
                                     {
                                         err = true;
-                                        println!("\x1b[G\x1b[K{}", s)
+                                        println!("\x1b[G\x1b[K{s}")
                                     }
                                     continue 'upper;
                                 }
@@ -500,7 +254,7 @@ fn main()
                                         add_var(le, r, i, &mut vars, options, false, false, false)
                                     {
                                         err = true;
-                                        println!("\x1b[G\x1b[K{}", s)
+                                        println!("\x1b[G\x1b[K{s}")
                                     }
                                     continue 'upper;
                                 }
@@ -509,7 +263,7 @@ fn main()
                                 add_var(le, r, 0, &mut vars, options, false, false, false)
                             {
                                 err = true;
-                                println!("\x1b[G\x1b[K{}", s)
+                                println!("\x1b[G\x1b[K{s}")
                             }
                         }
                     }
@@ -523,13 +277,13 @@ fn main()
         {
             if let Err(s) = file_opts(&mut options, &mut colors, &file_path, &vars, check, false)
             {
-                println!("{}", s);
+                println!("{s}");
                 std::process::exit(1);
             }
         }
         if let Err(s) = arg_opts(&mut options, &mut colors, &mut args, &vars, false)
         {
-            println!("{}", s);
+            println!("{s}");
             std::process::exit(1);
         }
     }
@@ -627,7 +381,7 @@ fn main()
                                 if let Err(s) =
                                     set_commands_or_vars(&mut colors, &mut options, &mut vars, s)
                                 {
-                                    println!("{}", s);
+                                    println!("{s}");
                                     continue 'main;
                                 }
                             }
@@ -741,10 +495,77 @@ fn main()
             let mut xxbool = false;
             let mut xxstart = false;
             let mut xxpos = 0;
+            let mut lastd = get_terminal_dimensions();
             loop
             {
                 let c = read_single_char();
                 let watch = Instant::now();
+                {
+                    let d = get_terminal_dimensions();
+                    if lastd != d
+                    {
+                        lastd = d;
+                        end = start + get_terminal_dimensions().0
+                            - if options.prompt { 3 } else { 1 };
+                        if end > input.len()
+                        {
+                            end = input.len()
+                        }
+                        if placement > end
+                        {
+                            placement = end
+                        }
+                        if options.real_time_output && !slow
+                        {
+                            execute!(stdout, DisableBlinking).unwrap();
+                            (frac, graphable, long, varcheck) = print_concurrent(
+                                &input,
+                                &last,
+                                &vars,
+                                options,
+                                colors.clone(),
+                                start,
+                                end,
+                                false,
+                            );
+                            if watch.elapsed().as_millis() > options.slowcheck
+                            {
+                                firstslow = true;
+                                slow = true;
+                            }
+                        }
+                        else if options.real_time_output && firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &vars,
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
+                        else
+                        {
+                            clearln(&input, &vars, start, end, options, &colors);
+                        }
+                        if options.debug
+                        {
+                            let time = watch.elapsed().as_nanos();
+                            print!(
+                                " {}\x1b[{}D",
+                                time,
+                                time.to_string().len() + 1 + end - placement
+                            );
+                        }
+                        else if end - placement != 0
+                        {
+                            print!("\x1b[{}D", end - placement)
+                        }
+                    }
+                }
                 match c
                 {
                     '\n' | '\x14' | '\x09' | '\x06' =>
@@ -784,7 +605,7 @@ fn main()
                         {
                             if !input.is_empty() && !input.starts_with(&['#']) && frac != 0
                             {
-                                print!("\x1b[{}B", frac);
+                                print!("\x1b[{frac}B");
                             }
                             if c == '\x14' || c == '\x06'
                             {
@@ -1594,7 +1415,7 @@ fn main()
                             else if placement == s
                             {
                                 placement = 0;
-                                print!("\x1b[{}D", s);
+                                print!("\x1b[{s}D");
                             }
                             else
                             {
@@ -1923,7 +1744,8 @@ fn main()
                                     "\x1b[G\x1b[{}A\x1b[{}C",
                                     n,
                                     placement + if options.prompt { 2 } else { 0 }
-                                )
+                                );
+                                long = true
                             }
                         }
                     }
@@ -1981,7 +1803,7 @@ fn main()
                                     else if placement == xxpos
                                     {
                                         placement = 0;
-                                        print!("\x1b[{}D", xxpos);
+                                        print!("\x1b[{xxpos}D");
                                     }
                                     else
                                     {
@@ -2432,6 +2254,29 @@ fn main()
         }
         else if graphable.graph
         {
+            if !options.gnuplot
+            {
+                if let Some(path) = find_it("kalc-plot")
+                {
+                    let data = Data {
+                        vars: vars.clone(),
+                        options,
+                    };
+                    handles.push(thread::spawn(move || {
+                        let mut plot = Command::new(path)
+                            .arg("-d")
+                            .arg(input.iter().collect::<String>())
+                            .stdin(Stdio::piped())
+                            .spawn()
+                            .unwrap();
+                        let stdin = plot.stdin.as_mut().unwrap();
+                        let data = serde_json::to_string(&data).unwrap();
+                        write!(stdin, "{data}").unwrap();
+                        plot.wait().unwrap();
+                    }));
+                    continue;
+                }
+            }
             let inputs: Vec<String> = insert_last(&input, &last.iter().collect::<String>())
                 .split('#')
                 .map(String::from)
@@ -2472,6 +2317,26 @@ fn main()
         }
     }
 }
+fn find_it<P>(exe_name: P) -> Option<PathBuf>
+where
+    P: AsRef<Path>,
+{
+    env::var_os("PATH").and_then(|paths| {
+        env::split_paths(&paths)
+            .filter_map(|dir| {
+                let full_path = dir.join(&exe_name);
+                if full_path.is_file()
+                {
+                    Some(full_path)
+                }
+                else
+                {
+                    None
+                }
+            })
+            .next()
+    })
+}
 fn setup_for_interactive(colors: &Colors, options: &mut Options, stdout: &mut Stdout)
 {
     options.interactive = true;
